from rmap.utils import exec_cmd, exec_cmd_bash, rmap_print_cmd, rmap_print_msg, get_ping_ttl
from colorama import Fore
from pathlib import Path
from time import sleep
import logging
import multiprocessing
import pexpect

logging.basicConfig(level=logging.DEBUG)
semaphore = multiprocessing.Semaphore(2)

class RMap:
    def __init__(self, host, debug, processes_limit, all_ports, arguments, vulnscan, ffuf_wordlist, ffuf_outtype, ffuf_recursion, ffuf_depth, ffuf_arguments):
        self.host = host
        self.ffuf_wordlist = ffuf_wordlist
        self.ffuf_outtype = ffuf_outtype
        self.all_ports = all_ports
        self.arguments = arguments
        self.vulnscan = vulnscan
        self.debug = debug
        self.processes_limit = processes_limit
        self.ffuf_recursion = ffuf_recursion
        self.ffuf_depth = ffuf_depth
        self.ffuf_arguments = ffuf_arguments


        if self.vulnscan:
            self.vulnscan()
        else:
            self.nmap()


    def vulnscan(self):

        exec_cmd(f"mkdir -p vulnscan")
        path = Path("/usr/share/nmap/scripts/vulscan")
        if not path.is_dir():
            vulscancmd = "git clone https://github.com/scipag/vulscan.git /usr/share/nmap/scripts/vulscan"
            rmap_print_msg("Nmap Vulnerability Scan", "Nmap Vulscan Install", vulscancmd)
            exec_cmd_bash(f"{vulscancmd} > /dev/null 2>&1")

        resultout2 = f"vulscan_{self.host}"
        nmapcmd = f'nmap -sV --script=vulscan/vulscan.nse -oN vulnscan/{resultout2} {self.host}'
        rmap_print_msg("Nmap Vulscan Vulnerability Scan", "EXEC", nmapcmd)
        exec_cmd(nmapcmd)

    def nmap(self):
        exec_cmd(f"mkdir -p nmap")

        resultout = f"{self.host}"

        if self.all_ports:
            cmdnmap = f"nmap {self.arguments} -p- -oA nmap/{resultout} {self.host}"
        else:
            cmdnmap = f"nmap {self.arguments} -oA nmap/{resultout} {self.host}"

        print(Fore.RED + "[*]" + Fore.MAGENTA + f' [EXEC] ' + Fore.RESET + Fore.BLUE + cmdnmap + Fore.RESET)
        exec_cmd(cmdnmap)

        xml_path = f"nmap/{resultout}.xml"

        services = self.parse_file(xml_path)
        self.analyse_nmap(services)


    def ffuf_dir_enum(self, port):
        with semaphore:
            sleep(1)
        exec_cmd(f"mkdir -p ffuf")
        resultout = f"ffuf_{self.host}:{port}"
        if self.ffuf_recursion:
            cmdffuf = f"ffuf -w {self.ffuf_wordlist} -u http://{self.host}:{port}/FUZZ -o ffuf/{resultout}.{self.ffuf_outtype} -of {self.ffuf_outtype} -debug-log ffuf/{resultout}.log -recursion {self.ffuf_recursion} -recursion-depth {self.ffuf_depth} {self.ffuf_arguments}"
        else:
            cmdffuf = f"ffuf -w {self.ffuf_wordlist} -u http://{self.host}:{port}/FUZZ -o ffuf/{resultout}.{self.ffuf_outtype}  -of {self.ffuf_outtype} -debug-log ffuf/{resultout}.log {self.ffuf_arguments}"
        rmap_print_cmd("HTTP", port, cmdffuf)
        exec_cmd(cmdffuf)
        if self.debug:
            logging.debug(f'[HTTP ENDED] {cmdffuf}')

    def nfs_enum(self, port):
        with semaphore:
            sleep(1)

        exec_cmd(f"mkdir -p nfs")

        resultout = f"nfs_{self.host}:{port}"
        cmdnmap = f"nmap --script=nfs-ls.nse,nfs-showmount.nse,nfs-statfs.nse -p {port} -oN nfs/{resultout} {self.host}"

        rmap_print_cmd("NFS", port, cmdnmap)
        exec_cmd(cmdnmap)
        if self.debug:
            logging.debug(f'[NFS ENDED] {cmdnmap}')        


    def smb_enum(self, port):
        with semaphore:
            sleep(1)
        exec_cmd(f"mkdir -p smb")

        resultout = f"smb_{self.host}:{port}"
        cmdnmap = f"nmap --script \"safe or smb-enum-*\" -p {port} -oN smb/{resultout} {self.host}"

        rmap_print_cmd("SMB", port, cmdnmap)
        exec_cmd(cmdnmap)
        if self.debug:
            logging.debug(f'[SMB NMAP ENDED] {cmdnmap}')
        
        resultout2 = f"smbclient_shares_{self.host}:{port}"
        smbclientcmd = f'''smbclient -N -L \\\\{self.host}'''
        print(Fore.RED + "[*]" + Fore.GREEN + f' [{port}] [SMB Shares]' + Fore.MAGENTA + f' [EXEC] ' + Fore.BLUE + smbclientcmd + Fore.RESET)
        exec_cmd_bash(f"{smbclientcmd} > smb/{resultout2}")
        if self.debug:
            logging.debug(f'[SMB Shares ENDED] {smbclientcmd}')

        resultout3 = f"smb_vuln_{self.host}:{port}"
        cmdnmap2 = f"nmap --script=smb-vuln-cve2009-3103.nse,smb-vuln-ms06-025.nse,smb-vuln-ms07-029.nse,smb-vuln-ms08-067.nse,smb-vuln-ms10-054.nse,smb-vuln-ms10-061.nse,smb-vuln-ms17-010.nse -p {port} -oN smb/{resultout3} {self.host}"
    
        print(Fore.RED + "[*]" + Fore.GREEN + f' [{port}] [SMB Vulnerability Check]' + Fore.MAGENTA + f' [EXEC] ' + Fore.BLUE + cmdnmap2 + Fore.RESET)
        exec_cmd(cmdnmap2)
        if self.debug:
            logging.debug(f'[SMB Vulnerability Check ENDED] {cmdnmap2}')
        

    def samba_enum(self, port):
        with semaphore:
            sleep(1)

        exec_cmd(f"mkdir -p smb")

        resultout = f"samba_{self.host}:{port}"
        cmdnmap = f"nmap --script=samba-vuln-cve-2012-1182 -p {port} -oN smb/{resultout} {self.host}"

        rmap_print_cmd("Samba", port, cmdnmap)
        exec_cmd(cmdnmap)
        if self.debug:
            logging.debug(f'[Samba NMAP ENDED] {cmdnmap}')


    def ftp_enum(self, port):
        with semaphore:
            sleep(1)
        exec_cmd(f"mkdir -p ftp")
        resultout = f"ftp_{self.host}:{port}"
        cmdnmap = f"nmap --script ftp-* -p {port} -oN ftp/{resultout} {self.host}"

        rmap_print_cmd("FTP", port, cmdnmap)
        exec_cmd(cmdnmap)
        if self.debug:
            logging.debug(f'[FTP ENDED] {cmdnmap}')


    def telnet_enum(self, port):
        with semaphore:
            sleep(1)
        exec_cmd(f"mkdir -p telnet")
        resultout = f"telnet_{self.host}:{port}"
        telnetnmap = f"nmap -n -sV -Pn --script \"*telnet* and safe\" -p {port} -oN telnet/{resultout} {self.host}"

        rmap_print_cmd("TELNET", port, telnetnmap)
        exec_cmd(telnetnmap)
        if self.debug:
            logging.debug(f'[TELNET ENDED] {telnetnmap}')

    def smtp_enum(self, port):
        with semaphore:
            sleep(1)
        exec_cmd(f"mkdir -p smtp")
        resultout = f"smtp_{self.host}:{port}"
        smtpnmap = f"nmap --script smtp-commands,smtp-open-relay -p {port} -oN smtp/{resultout} {self.host}"

        rmap_print_cmd("SMTP", port, smtpnmap)
        exec_cmd(smtpnmap)
        if self.debug:
            logging.debug(f'[SMTP ENDED] {smtpnmap}')

    def irc_enum(self, port):
        with semaphore:
            sleep(1)
        
        exec_cmd(f"mkdir -p irc")
        resultout = f"irc_{self.host}:{port}"
        ircnmap = f"nmap -sV --script irc-botnet-channels,irc-info,irc-unrealircd-backdoor -p {port} -oN irc/{resultout} {self.host}"

        rmap_print_cmd("IRC", port, ircnmap)
        exec_cmd(ircnmap)
        if self.debug:
            logging.debug(f'[IRC ENDED] {ircnmap}')

    def javarmi_enum(self, port):
        with semaphore:
            sleep(1)

        exec_cmd(f"mkdir -p javarmi")        
        resultout = f"javarmi_{self.host}:{port}"
        javarminmap = f"nmap -Pn -sV --script rmi-dumpregistry -p {port} -oN javarmi/{resultout} {self.host}"

        rmap_print_cmd("Java RMI", port, javarminmap)
        exec_cmd(javarminmap)
        if self.debug:
            logging.debug(f'[Java RMI ENDED] {javarminmap}')

    def ldap_enum(self, port):
        with semaphore:
            sleep(1)

        exec_cmd(f"mkdir -p ldap")
        resultout = f"ldap_{self.host}:{port}"
        ldapnmap = f"nmap -n -sV --script \"ldap* and not brute\" -p {port} -oN ldap/{resultout} {self.host}"

        rmap_print_cmd("LDAP", port, ldapnmap)
        exec_cmd(ldapnmap)

        if self.debug:
            logging.debug(f'[LDAP ENDED] {ldapnmap}')              

    def mysql_enum(self, port):
        with semaphore:
            sleep(1)

        exec_cmd(f"mkdir -p mysql")
        resultout = f"mysql_{self.host}:{port}"
        mysqlnmap = f"nmap -sV --script mysql-audit,mysql-databases,mysql-dump-hashes,mysql-empty-password,mysql-enum,mysql-info,mysql-query,mysql-users,mysql-variables,mysql-vuln-cve2012-2122 -p {port} -oN mysql/{resultout} {self.host}"

        rmap_print_cmd("MYSQL", port, mysqlnmap)
        exec_cmd(mysqlnmap)

        if self.debug:
            logging.debug(f'[MYSQL ENDED] {mysqlnmap}')

    def rdp_enum(self, port):
        with semaphore:
            sleep(1)

        exec_cmd(f"mkdir -p rdp")
        resultout = f"rdp_{self.host}:{port}"
        rdpnmap = f"nmap --script \"rdp-enum-encryption or rdp-vuln-ms12-020 or rdp-ntlm-info\" -p {port} -oN rdp/{resultout} {self.host}"

        rmap_print_cmd("RDP", port, rdpnmap)
        exec_cmd(rdpnmap)

        if self.debug:
            logging.debug(f'[RDP ENDED] {rdpnmap}')

    def redis_enum(self, port):
        with semaphore:
            sleep(1)

        exec_cmd(f"mkdir -p redis")
        resultout = f"redis_{self.host}:{port}"
        redisnmap = f"nmap --script redis-info -sV -p {port} -oN redis/{resultout} {self.host}"

        rmap_print_cmd("Redis", port, redisnmap)
        exec_cmd(redisnmap)

        if self.debug:
            logging.debug(f'[REDIS ENDED] {redisnmap}')
    
    def ajp13_enum(self, port):
        with semaphore:
            sleep(1)

        exec_cmd(f"mkdir -p ajp13")
        resultout = f"ajp13_{self.host}:{port}"
        ajp13nmap = f"nmap -sV --script ajp-auth,ajp-headers,ajp-methods,ajp-request -n -p {port} -oN ajp13/{resultout} {self.host}"

        rmap_print_cmd("Apache JServ", port, ajp13nmap)
        exec_cmd(ajp13nmap)

        if self.debug:
            logging.debug(f'[Apache JServ ENDED] {ajp13nmap}')                

    def couchdb_enum(self, port):
        with semaphore:
            sleep(1)

        exec_cmd(f"mkdir -p couchdb")
        resultout = f"couchdb_{self.host}:{port}"
        couchdbnmap = f"nmap -sV --script couchdb-databases,couchdb-stats -p {port} -oN couchdb/{resultout} {self.host}"

        print(Fore.RED + "[*]" + Fore.GREEN + f' [{port}] [CouchDB DETECTED]' + Fore.MAGENTA + f' [EXEC] ' + Fore.BLUE + couchdbnmap + Fore.RESET)
        rmap_print_cmd("CouchDB", port, couchdbnmap)
        exec_cmd(couchdbnmap)

        if self.debug:
            logging.debug(f'[CouchDB ENDED] {couchdbnmap}')        

    def bitcoin_enum(self, port):
        with semaphore:
            sleep(1)

        exec_cmd(f"mkdir -p bitcoin")
        resultout = f"bitcoin_{self.host}:{port}"
        btcnmap = f"nmap -sV --script bitcoin-info --script bitcoin-getaddr -p {port} -oN bitcoin/{resultout} {self.host}"

        rmap_print_cmd("Bitcoin", port, btcnmap)
        exec_cmd(btcnmap)

        if self.debug:
            logging.debug(f'[Bitcoin ENDED] {btcnmap}')        


    def cassandra_enum(self, port):
        with semaphore:
            sleep(1)

        exec_cmd(f"mkdir -p cassandra")
        resultout = f"cassandra_{self.host}:{port}"
        cassandranmap = f"nmap -sV --script cassandra-info -p {port} -oN cassandra/{resultout} {self.host}"

        rmap_print_cmd("Cassandra", port, cassandranmap)
        exec_cmd(cassandranmap)

        if self.debug:
            logging.debug(f'[Cassandra ENDED] {cassandranmap}')        


    def mongodb_enum(self, port):
        with semaphore:
            sleep(1)

        exec_cmd(f"mkdir -p mongodb")
        resultout = f"mongodb_{self.host}:{port}"
        mongodbnmap = f"nmap -sV --script \"mongo* and default\" -p {port} -oN mongodb/{resultout} {self.host}"

        rmap_print_cmd("MongoDB", port, mongodbnmap)
        exec_cmd(mongodbnmap)

        if self.debug:
            logging.debug(f'[MongoDB ENDED] {mongodbnmap}')

    def pop3_enum(self, port):
        with semaphore:
            sleep(1)

        exec_cmd(f"mkdir -p pop3")
        resultout = f"pop3_{self.host}:{port}"
        pop3nmap = f"nmap -sV --script \"pop3-capabilities or pop3-ntlm-info\" -p {port} -oN pop3/{resultout} {self.host}"

        rmap_print_cmd("POP3", port, pop3nmap)
        exec_cmd(pop3nmap)

    def amqp_enum(self, port):
        with semaphore:
            sleep(1)

        exec_cmd(f"mkdir -p amqp")
        resultout = f"amqp_{self.host}:{port}"
        amqpnmap = f"nmap -sV --script amqp-info -p {port} -oN amqp/{resultout} {self.host}"

        rmap_print_cmd("RabbitMQ", port, amqpnmap)
        exec_cmd(amqpnmap)


from rmap.scan import RMap
from libnmap.parser import NmapParser
import multiprocessing


class NmapHandler(RMap):
    def __init__(self, host, debug, processes_limit, all_ports, arguments, vulnscan, ffuf_wordlist, ffuf_outtype, ffuf_recursion, ffuf_depth, ffuf_arguments):
        super().__init__(host, debug, processes_limit, all_ports, arguments, vulnscan, ffuf_wordlist, ffuf_outtype, ffuf_recursion, ffuf_depth, ffuf_arguments)
        
    def parse_file(self, path_xml):
        report = NmapParser.parse_fromfile(path_xml)
        
        services = []

        for host in report.hosts:
            for serv in host.services:
                services.append(f"{serv.port}:{serv.service}:{serv.banner}")
                
        return services

    def analyse_nmap(self, services):
        with multiprocessing.Pool(processes=int(self.processes_limit)) as pool:
            for service in services:
                service = service.split(":")
                ### FTP
                if service[1] == "ftp":
                    pool.apply_async(self.ftp_enum, [service[0]])
                ### TELNET
                if service[1] == "telnet":
                    pool.apply_async(self.telnet_enum, [service[0]])
                ### HTTP
                if service[1] == "http":
                    pool.apply_async(self.ffuf_dir_enum, [service[0]])
                ### SMTP
                if service[1] == "smtp":
                    pool.apply_async(self.smtp_enum, [service[0]])
                ### RDP
                if service[1] == "ms-wbt-server":
                    pool.apply_async(self.rdp_enum, [service[0]])
                ### IRC
                if service[1] == "irc":
                    pool.apply_async(self.irc_enum, [service[0]])
                ### Redis
                if service[1] == "redis":
                    pool.apply_async(self.redis_enum, [service[0]])
                ### LDAP
                if service[1] == "ldap":
                    pool.apply_async(self.ldap_enum, [service[0]])
                ### MYSQL
                if service[1] == "mysql":
                    pool.apply_async(self.mysql_enum, [service[0]])
                ### Apache JServ
                if service[1] == "ajp13":
                    pool.apply_async(self.ajp13_enum, [service[0]])
                ### Java RMI
                if service[1] == "java-rmi":
                    pool.apply_async(self.javarmi_enum, [service[0]])
                ### MongoDB
                if service[1] == "mongodb":
                    pool.apply_async(self.mongodb_enum, [service[0]])
                ### POP3
                if service[1] == "pop3":
                    pool.apply_async(self.pop3_enum, [service[0]])
                ### NFS
                if service[1] == "nfs":
                    pool.apply_async(self.nfs_enum, [service[0]])
                ### AMQP (advanced message queuing protocol)
                if service[1] == "amqp":
                    pool.apply_async(self.amqp_enum, [service[0]])
                ### SMB
                if int(service[0]) == 445:
                    pool.apply_async(self.smb_enum, [service[0]])
                ### Samba
                if "Samba" in str(service[2]) and str(service[1]) == "netbios-ssn":
                    pool.apply_async(self.samba_enum, [service[0]])
                ### CouchDB
                if int(service[0]) == 5984 or int(service[0]) == 6984:
                    pool.apply_async(self.couchdb_enum, [service[0]])

            pool.close()
            pool.join()



import pexpect
import uuid
import os
from colorama import Fore
import subprocess
import re

exec_timeout = 600

def rmap_print_cmd(proto, port, cmd):
    print(Fore.RED + "[*]" + Fore.GREEN + f' [{port}] [{proto} DETECTED]' + Fore.MAGENTA + f' [EXEC] ' + Fore.BLUE + cmd + Fore.RESET)

def rmap_print_msg(label, op, msg):
    print(Fore.RED + "[*]" + Fore.GREEN + f' [{label}]' + Fore.MAGENTA + f' [{op}] ' + Fore.BLUE + msg + Fore.RESET)

def rmap_print_timeout(cmd):
    print(Fore.RED + "[*]" + Fore.RED + f' [TIMEOUT]' + Fore.MAGENTA + f' [EXIT] ' + Fore.RED + cmd + Fore.RESET)


def is_tool(name):
    try:
        devnull = open(os.devnull)
        subprocess.Popen([name], stdout=devnull, stderr=devnull).communicate()
    except OSError as e:
        if e.errno == errno.ENOENT:
            return False
    return True


def get_ping_ttl(host):

    p = subprocess.Popen(["ping", "-c 1", host], stdout=subprocess.PIPE)
    res = p.communicate()[0]
    if p.returncode > 0:
        return 0
    else:
        pattern = re.compile(r'[t,T][t,T][l,L]=\d*')
        ttl_group = pattern.search(str(res)).group()
        result_ttl = re.findall(r'\d+', ttl_group)

        return int(result_ttl[0])


def check_ping(ip):
    response = os.system("ping -c 1 " + ip + " > /dev/null")
    # and then check the response...
    if response == 0:
        pingstatus = True
    else:
        pingstatus = False
    
    return pingstatus


def exec_cmd(cmd_str):
    return pexpect.run(cmd_str, encoding='utf-8', timeout=exec_timeout)


def exec_cmd_bash(cmd_str):
    p = pexpect.spawn('/bin/bash', ['-c', cmd_str], encoding='utf-8')
    p.expect(pexpect.EOF, exec_timeout)

    p.close()
    return p.before, p.exitstatus


def hex_uuid():
    return uuid.uuid4().hex


#!/usr/bin/python

from rmap import __version__
from rmap.nmap import NmapHandler
from rmap.banner import banner
from rmap.utils import check_ping, exec_cmd, is_tool
from configparser import ConfigParser
from colorama import Fore
import urllib.request
import argparse
import subprocess
import os
from pathlib import Path
import sys
import errno
import logging

if sys.version_info.major < 3:
    print("RMap supports only Python3. Rerun application in Python3 environment.")
    exit(0)

if os.geteuid() != 0:
    print("RMap must run as root. Rerun application using sudo.")
    exit(0)

if not is_tool("nmap"):
    print("Must have nmap installed.")
    exit(0)

if not is_tool("ffuf"):
    print("Must have ffuf installed.")
    exit(0)

if not is_tool("smbclient"):
    print("Must have smbclient installed.")
    exit(0)

def init():
    parser = argparse.ArgumentParser()

    #parser.add_argument('', '--ip', type=str, required=True, help="IP Address")
    parser.add_argument('ip', help="Target IP Address", type=str)
    parser.add_argument('--vuln', default=False, help="Scan host for vulnerabilities", action="store_true")
    parser.add_argument('-d', '--debug', default=False, help="Debug output", action="store_true")
    parser.add_argument('-v', '--version', help="Show version", action='version', version=__version__)

    args = parser.parse_args()
    
    return args

def main():
    args = init()
    banner()
    
    if not check_ping(args.ip):
        logging.error("Did not pass ping check.")
        sys.exit()
    
    exec_cmd("mkdir -p /usr/share/rmap")
    path = Path("/usr/share/rmap/rmap.conf")

    if not path.is_file():
        urllib.request.urlretrieve("https://raw.githubusercontent.com/syspuke/rmap/dev/rmap.conf", "/usr/share/rmap/rmap.conf")
    
    try:
        # Config parser
        config_object = ConfigParser()

        config_object.read("/usr/share/rmap/rmap.conf")

        processes_limit = config_object["rmap"]["processLimit"]
        nmap_all_ports = config_object["nmap"]["allports"]
        nmap_arguments = config_object["nmap"]["arguments"]
        ffuf_wordlist = config_object["ffuf"]["wordlist"]
        ffuf_outtype = config_object["ffuf"]["outtype"]
        ffuf_recursion = config_object["ffuf"]["recursion"]
        ffuf_depth = config_object["ffuf"]["recursionDepth"]
        ffuf_arguments = config_object["ffuf"]["arguments"]

        if nmap_all_ports == "false":
            nmap_all_ports = False
        else:
            nmap_all_ports = True
        if ffuf_recursion == "false":
            ffuf_recursion = False
        else:
            ffuf_recursion = True
    except (IOError, KeyError) as err:
        logging.error(f"Config error: {err}. Using default values.")
        processes_limit = 2
        nmap_all_ports = False
        nmap_arguments = "-sC -sV"
        ffuf_wordlist = "/usr/share/seclists/Discovery/Web-Content/big.txt"
        ffuf_outtype = "md"
        ffuf_recursion = True
        ffuf_depth = 1
        ffuf_arguments = "-fc 302"
    
    try:
        NmapHandler(args.ip, args.debug, int(processes_limit), nmap_all_ports, nmap_arguments, args.vuln, ffuf_wordlist, ffuf_outtype, ffuf_recursion, ffuf_depth, ffuf_arguments)
    except KeyboardInterrupt:
        print(Fore.RED + "[*]" + Fore.YELLOW + f' [CTRL + C DETECTED]' + Fore.RED + f' [EXITING] ' + Fore.RESET)
        sys.exit(0)

if __name__ == "__main__":
    main()


from setuptools import setup
from rmap import __version__

setup(
    name = 'python-rmap',
    description='Automated enumeration for red teamers',
    license = 'gpl-3.0',
    author = "syspuke",
    author_email='syspuke@pm.me',
    url='https://github.com/syspuke/rmap',
    version = __version__,
    packages = ['rmap'],
    entry_points = {
        'console_scripts': [
            'rmap = rmap.main:main'
        ]
    },
    python_requires='>=3.6',
    )